#import "BWBorkSortPreferences.h"

#import "BWLog.h"

static const char *cvsid = "$Id: BWBorkSortPreferences.m,v 1.1.1.1 2002/03/25 01:36:47 markd Exp $";
static BWLogModule *logger;

static NSString *BWKeepFolder = @"KeepFolder";
static NSString *BWKeepType = @"KeepType";


@implementation BWBorkSortPreferences : NSObject

+ (void) initialize
{
    [super initialize];

    logger = [BWLogModule registerModule: @"BWBorkSortPreferences"
			  withCVSId: cvsid];

} // initialize


+ (NSMutableDictionary *) makeFactoryDefaults
{
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    NSString *homeDirectory;
    NSString *documentDirectory;

    homeDirectory = NSHomeDirectory ();

    // !!! must be a better way
    documentDirectory = [homeDirectory stringByAppendingString: @"/Documents"];

    [defaultValues setObject: documentDirectory
		   forKey: BWKeepFolder];

    [defaultValues setObject: [NSNumber numberWithInt: BWCopy]
		   forKey: BWKeepType];

    BWLog (logger, BWTrace, @"makeFactoryDefaults: %@", defaultValues);

    return (defaultValues);

} // makeFactoryDefaults


+ (BWBorkSortPreferences *) defaultPreferences
{
    static BWBorkSortPreferences *preferences = nil;

    if (preferences == nil) {
	NSMutableDictionary *defaultValues = [self makeFactoryDefaults];
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults registerDefaults: defaultValues];
	preferences = [[BWBorkSortPreferences alloc] init];
    }

    BWLog (logger, BWTrace, @"default preferences: %@", preferences);

    return (preferences);

} // defaultPreferences


- (NSString *) description
{
    NSString *description;

    description = [NSString stringWithFormat: @"keepFolder: %@, keepType: %d",
			    [self currentKeepFolder], [self keepType]];

    return (description);
    
} // description


- (void) setCurrentKeepFolder: (NSString *) path
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

    BWLog (logger, BWTrace, @"@set keep folder from: %@  to: %@",
	   [self currentKeepFolder], path);
    
    // ??? verify path?
    [defaults setObject: path  forKey: BWKeepFolder];

} // setCurrentKeepFolder


- (NSString *) currentKeepFolder
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *currentKeepFolder;

    currentKeepFolder = [defaults objectForKey: BWKeepFolder];

    return (currentKeepFolder);
    
} // currentKeepFolder


- (void) setKeepType: (BWBorkSortKeepType) type
{
    if ((type != BWCopy) && (type != BWMove)) {
	BWLog (logger, BWError, 
	       @"setKeepType: Unexpected value: %d.  not changing preferences");

    } else {
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSNumber *number;

	BWLog (logger, BWTrace, @"@set keep type from: %d  to: %d",
	       [self keepType], type);

	number = [NSNumber numberWithInt: type];
	[defaults setObject: number  forKey: BWKeepType];
    }

} // setRemoveOriginal



- (BWBorkSortKeepType) keepType
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSNumber *keepType;
    BWBorkSortKeepType type;

    keepType = [defaults objectForKey: BWKeepType];
    type = [keepType intValue];

    // in case someone diddled
    if (type != BWCopy && type != BWMove) {
	// !!! BWLog This
	NSLog (@"bad default keep type: %d.  Defaulting to 'copy'", type);
	type = BWCopy;
    }

    return (type);

} // removeOriginal


@end // BWBorkSortPreferences

