#import "BWLog.h"

static NSMutableDictionary *g_modules;


static NSMutableDictionary *getModules ()
{
    if (g_modules == nil) {
	g_modules = [NSMutableDictionary dictionaryWithCapacity: 200];
	[g_modules retain];
    }

    return (g_modules);

} // getModules


@implementation BWLogModule

+ (NSArray *) getRegisteredModules
{
    NSMutableDictionary *modules = getModules ();
    NSArray *result;

    result = [modules allValues];

    return (result);

} // getRegiteredModules


+ (BWLogModule *) getModuleByName: (NSString *) name
{
    BWLogModule *module;
    NSMutableDictionary *modules = getModules ();

    module = [modules objectForKey: name];

    return (module);

} // getModuleByName


+ (BWLogModule *) registerModule: (NSString *) name
{
    return ([BWLogModule registerModule: name withCVSId: NULL]);
} // registerModule


+ (BWLogModule *) registerModule: (NSString *) name  
	withCVSId: (const char *) cvsIdstring
{
    BWLogModule *module;
    NSMutableDictionary *modules = getModules ();

    module = [modules objectForKey: name];

    if (module != nil) {
	// is this really exceptional enough to warrant a gripe?
	NSLog (@"module %@ already registered.", name);
	return (module);
    }

    module = [[BWLogModule alloc] init];

    module->moduleName = name;
    module->cvsId = cvsIdstring;
    [module enableLogging];
    [module setLogLevel: BWDefaultLogLevel];
    [module retain];

    [modules setObject: module  forKey: name];

    return (module);

} // registerModule: withCVSId


- (void) enableLogging
{
    loggingEnabled = YES;
} // enableLogging


- (void) disableLogging
{
    loggingEnabled = NO;
} // disableLogging


- (BOOL) isLogging
{
    return (loggingEnabled);
} // isLogging


// threshold of when to log vs when to just eat the logging statements
- (void) setLogLevel: (BWLogLevel) level
{
    logLevel = level;
} // setLogLevel


- (BWLogLevel) logLevel
{
    return (logLevel);
} // logLevel


- (NSString *) name
{
    return (moduleName);
} // name


- (const char *) CVSId
{
    return (cvsId);
} // CVSId


//
// ----- HouseKeeping
//

- (NSString *) description
{
    NSString *x;
    const char *loggingP;
    
    loggingP = (loggingEnabled) ? "enabled" : "disabled";

    x = [NSString stringWithFormat: 
		      @"BWLogModule: %@ logging: %s level: %d cvsId: %s",
		  moduleName, loggingP, logLevel, cvsId];
    return (x);

} // description


@end // BWLog


//
// ----- Wastin' away again in C-Ville
//

// why in C and not an object method? 
//   * to avoid the (low, but non-zero) cost of a method invocation
//   * this might be changable to a macro to avoid any function call

void BWLog (BWLogModule *module, BWLogLevel level, NSString *format, ...)
{
    if (module == nil) {
	// uh.....
	NSLog (@"BWLog: oops!  Given a null module");
	return;
    }

    if (level <= module->logLevel) {
	va_list args;
	NSString *logString;
	
	va_start (args, format);
	logString = [[NSString alloc] initWithFormat: format  arguments: args];
	va_end (args);

	// feature creep - delegate to some other object
	NSLog (@"%@: %@", [module name], logString);
	[logString release];
    }

    if (level <= BWFatal) {
	abort ();
    }
    
} // BWLog

